(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

open Expressions
open Types
open Declarations

type axioms = declaration list * expression list

(* (is_axiom a) checks that a is an axiom, that is an equality between *)
(* two functional expressions *)
let is_axiom = function
    Infix("=",l,r) -> true
  | _ -> false


exception Exc_axioms of string
let check_axioms sign (declarations,expressions) =
  begin
    (* check that all the declarations are well-formed *)
    check_declarations declarations;

    (* check that all variable declarations are logical *)
    List.iter
      (function (x,(k,_)) -> match k with
	   Mutable -> raise
	     (Exc_axioms ("variable \"" ^ x ^ "\" declared " 
			  ^ "as mutable in axiom section"))
	 | Logical -> ()
	 | ProcessKind -> failwith "this cannot happen: processkind in axiom")
      declarations;

    (* check that both sides of the equations are well-typed *)
    (* and have the same type.                               *)
    List.iter
      (function expr ->	match expr with
	   Infix("=",l,r) ->
	     let lt = 
	       try 
		 type_expr sign declarations [] [] l
	       with
		   Exc_typeexpression s -> raise
		     (Exc_axioms
			("expression \"" ^ (string_of_expression l) ^
			 "\" cannot be typed because " ^ s))
	     and rt =
	       try
		 type_expr sign declarations [] [] r
	       with
		   Exc_typeexpression s -> raise
		     (Exc_axioms
			("expression \"" ^ (string_of_expression r) ^
			 "\" cannot be typed because " ^ s)) 
	     in if not (has_inf_type lt  rt)
	       then (raise
		       (Exc_axioms
			  ("axiomatic equation between expressions "
			   ^ "of inconsistentg type:\n" ^
			   "left side \"" ^ (string_of_expression l) ^
			   "\" has type " ^ (string_of_type_expression lt) ^ 
			   "\nright side \"" ^ (string_of_expression r) ^
			   "\" has type " ^
			   (string_of_type_expression rt ))))
	 | _ -> failwith "this cannot happen: non-equational axiom") 
      expressions
  end
    
