(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(**************************************************************************)
(* Equational Axioms                                                      *)
(**************************************************************************)

(* the parser guarantees that every expression in the second component *)
(* of an element of type "axiom" passes "is_axiom", that is that it is *)
(* an equation between two functional expressions.                     *)
type axioms = Declarations.declaration list * Expressions.expression list

(* (is_axiom a) checks that a is an axiom, that is an equality between *)
(* two functional expressions *)
val is_axiom: Expressions.expression -> bool

(* check that axioms is well-formed:                                      *)
(* - the declaration list must be well-formed                             *)
(* - no declaration may be mutable                                        *)
(* - both sides of an equation have the same type                         *)
exception Exc_axioms of string
val check_axioms: Signatures.simple_signature -> axioms -> unit

