(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(***********************************************)
(*                     Scenarios               *)
(***********************************************)

open Types
open Intervals
open Expressions
open Declarations

(* raw abstract syntax. *)
type rawscenario =
    SRawLabeled of string * rawscenario
  | SRawCall of string * expression list
  | SRawNew of expression
  | SRawForever of rawscenario list
  | SRawParallel of rawscenario list
  | SRawSequence of rawscenario list
  | SRawAssign of expression * expression
  | SRawConditional of expression * rawscenario list * rawscenario list
  | SRawForall of declaration list * interval list * rawscenario
  | SRawExists of declaration list * rawscenario

(* processed abstract syntax. The first argument is the label of a   *)
(* sub-scenario.                                                     *)
type scenario =
    SCall of string * string * expression list
  | SNew of string * expression
  | SForever of string * scenario list
  | SParallel of string * scenario list
  | SSequence of string * scenario list
  | SAssign of string * expression * expression
  | SConditional of string * expression * scenario list * scenario list
  | SForall of string * declaration list * interval list * scenario
  | SExists of string * declaration list * scenario

(* process a scenario. The string argument is the prefix         *)
(* used in the generation of default labels for sub-scenarios.   *)
val process_scenario: string -> rawscenario -> scenario

(* (print_scenario oc s) prints scenario s to the output channel oc *)
val print_scenario: out_channel -> scenario -> unit

(* collect a sorted list without muliple occurrences of all numerical  *)
(* constants used in a scenario.                                       *)
val int_constants: scenario -> int list

(* type-check a scenario for a signature, a list of role definitions, *)
(* and a list of global variable declarations.                        *)
exception Exc_scenario of string
val check_types_scenario:
  Signatures.simple_signature ->
  (string * (Declarations.declaration list)) list ->
  (Declarations.sk_declaration list) ->
  scenario ->
  unit

(* (gather_checkpoints scenario) returns the list of checkpoints *)
(* defined in scenario.                                          *)
val gather_checkpoints: scenario -> string list

