(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

open Expressions

type trs = ( expression * expression ) list

let trs_to_string =
  List.fold_left 
    (fun s (l,r) -> s ^
       (string_of_expression l) ^ " -> " ^
       (string_of_expression r) ^ "\n")
    ""

type substitution = (string * expression) list

let rec apply_subst subst = function
    Ident s -> 
      (try List.assoc s subst
       with
	   Not_found -> Ident s)
  | Int i -> Int i
  | Apply(f,tl) -> Apply(f,List.map (apply_subst subst) tl)
  | Infix(op,e1,e2) -> Infix(op,
			     (apply_subst subst e1),
			     (apply_subst subst e2))
  | Inverse e -> Inverse (apply_subst subst e)
  | Tuple el -> Tuple(List.map (apply_subst subst) el)
  | Liste el -> Liste(List.map (apply_subst subst) el)
  | AssocList pl ->
      AssocList(List.map
		  (function (e1,e2) -> (apply_subst subst e1,
					apply_subst subst e2))
		  pl)





