(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(*******************************************************************)
(*      Module Utils: some utilitarian functions                   *)

(*******************************************************************)

(* (singlyfy l) returns a sorted version of l without repetitions *)
val singlyfy: 'a list -> 'a list

(* convert a list of strings into one string, by listing them seperated  *)
(* by the string separator                                               *)
val string_of_string_list: string -> string list -> string

(* (alldifferent l) checks that all elements (strings) of l are different *)
(* If a string s occurs twice in l then the exception Twice(s) is raised. *)
exception Twice of string
val alldifferent: string list -> unit

(* (listminus cmp l1 l2), where cmp is a compare function and l1 and l2 *)
(* are sorted according to cmp, returns (l1-l2)                         *)
val listminus: ('a -> 'a -> int) -> 'a list -> 'a list -> 'a list

(* mapindex is like map, except that the function f takes as additional *)
(* first argument the number of the element in the list (starting on 0) *)
(* Example: (mapindex f [a;b;c]) yields [(f 0 a); (f 1 b); (f 2 c)]     *)
val mapindex : (int -> 'a -> 'b) -> 'a list -> 'b list

(* (checksubset l1 l2) tests whether every element of l1 is an element  *)
(* of l2. If this is the case it returns unit, otherwise it raises the  *)
(* excpetion Missing(e) where e is the first element of l1 which is not *)
(* in l2.                                                               *)
exception Missing of string
val checksubset : string list -> string list -> unit
